package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysUserMapper
import com.gxzc.zen.api.sys.model.SysUser
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.cache.CacheManager
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service

/**
 * <p>
 * 用户管理 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysUserServiceImpl : ServiceImpl<SysUserMapper, SysUser>(), ISysUserService {
    companion object {
        const val CACHE_KEY_ALL = "all"
    }

    @Autowired
    private lateinit var cacheManager: CacheManager

    //    @PostConstruct
    fun initCache() {
        getListCacheable()
    }


    @Cacheable(CACHEKEYS.USER, key = "'$CACHE_KEY_ALL'")
    override fun getListCacheable(): MutableList<SysUser> {
        return baseMapper.selectByParams(null)
    }

    override fun getUserByAccountCacheable(account: String): SysUser? {
        return getListCacheable().find {
            it.account == account
        }
    }

    override fun getUserByIdCacheable(id: Long): SysUser? {
        return getListCacheable().find {
            it.id == id
        }
    }

    @Suppress("UNCHECKED_CAST")
    @ZenTransactional
    override fun insertCacheable(entity: SysUser) {
        if (baseMapper.insert(entity) == 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        val cache = cacheManager.getCache(CACHEKEYS.USER)
        val cachedList: MutableList<SysUser>? = cache[CACHE_KEY_ALL].get() as MutableList<SysUser>?
        cachedList?.add(entity)
    }

    @Suppress("UNCHECKED_CAST")
    @ZenTransactional
    override fun modify(entity: SysUser): SysUser {
        baseMapper.updateNoLogic(entity, EntityWrapper<SysUser>().eq("id", entity.id))
        // 更新缓存
        val cache = cacheManager.getCache(CACHEKEYS.USER)
        val cachedList: MutableList<SysUser>? = cache[CACHE_KEY_ALL].get() as MutableList<SysUser>?
        cachedList?.let {
            val idx = it.indexOfFirst { it.id == entity.id }
            if (idx != -1) {
                it[idx] = entity
            }
        }
        return entity
    }

    @Suppress("UNCHECKED_CAST")
    @ZenTransactional
    override fun physicalDeleteCacheable(id: Long) {
        if (baseMapper.physicalDelete(EntityWrapper<SysUser>().eq("id", id)) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        //
        val cache = cacheManager.getCache(CACHEKEYS.USER)
        val cachedList: MutableList<SysUser>? = cache[CACHE_KEY_ALL].get() as MutableList<SysUser>?
        cachedList?.let {
            it.removeIf {
                it.id == id
            }
        }
    }
}
