package com.gxzc.zen.common.config.response

import com.fasterxml.jackson.databind.ObjectMapper
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilters
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.core.MethodParameter
import org.springframework.http.MediaType
import org.springframework.http.converter.HttpMessageConverter
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter
import org.springframework.http.converter.json.MappingJacksonValue
import org.springframework.http.server.ServerHttpRequest
import org.springframework.http.server.ServerHttpResponse
import org.springframework.web.bind.annotation.ControllerAdvice
import org.springframework.web.servlet.mvc.method.annotation.AbstractMappingJacksonResponseBodyAdvice

/**
 * Controller aop 切面拦截写入body时动态设置filter
 * @author NorthLan
 * @date 2018/3/13
 * @url https://noahlan.com
 */
@ControllerAdvice
class DynamicFilterResponseBodyAdvice : AbstractMappingJacksonResponseBodyAdvice() {

    @Autowired
    @Qualifier(ZenJsonConfigurer.CONVERTER_NAME)
    private lateinit var converter: MappingJackson2HttpMessageConverter

    override fun supports(returnType: MethodParameter, converterType: Class<out HttpMessageConverter<*>>): Boolean {
        converter.objectMapper = ObjectMapper()
        return super.supports(returnType, converterType) &&
                (returnType.getMethodAnnotation(ZenResponseFilter::class.java) != null
                        || returnType.getMethodAnnotation(ZenResponseFilters::class.java) != null)
    }

    override fun beforeBodyWriteInternal(bodyContainer: MappingJacksonValue, contentType: MediaType, returnType: MethodParameter, request: ServerHttpRequest, response: ServerHttpResponse) {
        val jsonSerializer = ZenJsonSerializer()
        returnType.methodAnnotations.forEach {
            if (it is ZenResponseFilter) {
                jsonSerializer.filter(it)
            } else if (it is ZenResponseFilters) {
                it.value.forEach {
                    jsonSerializer.filter(it)
                }
            }
        }
        bodyContainer.filters = jsonSerializer.jacksonFilter
        converter.objectMapper = jsonSerializer.mapper
    }
}