package com.gxzc.zen.orm

import com.gxzc.zen.orm.contants.DSKey
import org.slf4j.LoggerFactory
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.jdbc.datasource.lookup.AbstractRoutingDataSource

/**
 * 动态数据源ThreadLocal
 * @author NorthLan at 2018/1/29
 */
@ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
class DynamicMultipleDataSource : AbstractRoutingDataSource() {
    init {
        logger.info("DynamicMultipleDataSource initializing...")
        DATA_SOURCE_KEY.set(DSKey.DSKEY_SYS)
    }

    companion object {
        private val logger = LoggerFactory.getLogger(DynamicMultipleDataSource::class.java)
        private val DATA_SOURCE_KEY = ThreadLocal<String>()

        fun setDataSource(key: String) {
            DATA_SOURCE_KEY.set(key)
        }

        fun getDataSource(): String {
            return DATA_SOURCE_KEY.get()
        }

        fun clear() {
            DATA_SOURCE_KEY.remove()
        }
    }

    override fun determineCurrentLookupKey(): Any {
        return DATA_SOURCE_KEY.get()
//        clear()
//        return lookupKey
    }
}