package com.gxzc.zen.orm.config

import com.alibaba.druid.pool.xa.DruidXADataSource
import com.baomidou.mybatisplus.spring.MybatisSqlSessionFactoryBean
import com.baomidou.mybatisplus.spring.boot.starter.MybatisPlusProperties
import com.baomidou.mybatisplus.spring.boot.starter.SpringBootVFS
import com.gxzc.zen.orm.contants.DSKey
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.boot.jta.atomikos.AtomikosDataSourceBean
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.DependsOn
import org.springframework.context.annotation.Primary
import javax.sql.DataSource
import javax.sql.XADataSource

/**
 *
 * @author NorthLan
 * @date 2018/3/24
 * @url https://noahlan.com
 */
@Configuration
@EnableConfigurationProperties(MybatisPlusProperties::class)
@ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "false")
class SingleDataSourceConfig {
    companion object {
        private val logger = LoggerFactory.getLogger(MultipleDataSourceConfig::class.java)
    }

    init {
        logger.info("SingleDataSourceConfig initializing...")
    }

    @Autowired
    private lateinit var properties: MybatisPlusProperties

    @Bean(DSKey.DSKEY_SYS + "druid")
    @ConfigurationProperties(prefix = "datasource.sys")
    fun dataSourceSysDruid(): XADataSource {
        return DruidXADataSource()
    }

    @Bean(DSKey.DSKEY_SYS)
    @DependsOn(DSKey.DSKEY_SYS + "druid")
    @Primary
    fun dataSourceSys(): DataSource {
        return AtomikosDataSourceBean().also {
            it.xaDataSource = dataSourceSysDruid()
        }
    }

    @Bean
    @Primary
    fun mybatisSqlSessionFactoryBean(@Qualifier(DSKey.DSKEY_SYS) dataSource: DataSource): MybatisSqlSessionFactoryBean {
        return MybatisSqlSessionFactoryBean().also {
            it.setDataSource(dataSource)
            it.vfs = SpringBootVFS::class.java
            it.setConfiguration(properties.configuration)
            it.setGlobalConfig(properties.globalConfig?.convertGlobalConfiguration())
            it.setMapperLocations(properties.resolveMapperLocations())
        }
    }
}