package com.gxzc.zen.web.sys.controller

import com.baomidou.kisso.SSOConfig
import com.baomidou.kisso.SSOHelper
import com.baomidou.kisso.SSOToken
import com.baomidou.kisso.annotation.Action
import com.baomidou.kisso.annotation.Login
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.dto.RequestDto
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.umps.util.MD5Salt
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

/**
 * 身份验证相关控制器
 * @author NorthLan
 * @date 2018/2/10
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("auth")
class AuthController : BaseController() {
    @Autowired
    private lateinit var userService: ISysUserService

    @Login(action = Action.Skip)
    @ApiOperation(value = "登录")
    @PostMapping("/login")
    fun login(@RequestBody data: RequestDto): ResponseEntity<*> {
        // 验证输入合法性
        val account = data["account"]?.toString()?.trim()
        val password = data["password"]?.toString()
        val rememberMe = data["rememberMe"]?.let { it as Boolean }

        if (account.isNullOrEmpty() || password.isNullOrEmpty()) {
            throw ZenException(ZenExceptionEnum.REQUEST_NULL)
        }
        // 验证账号密码
        val user = userService.getUserByAccountCacheable(account!!)
                ?: throw ZenException(ZenExceptionEnum.AUTH_ACCOUNT_NOT_EXISTS)
        // 对密码进行盐值处理比对
        if (user.password != MD5Salt.md5SaltEncode(user.salt!!, password!!)) {
            throw ZenException(ZenExceptionEnum.AUTH_PASSWORD_ERROR)
        }

        // 生成登陆 token->cookie
        if (rememberMe != null && rememberMe) {
            SSOConfig.getInstance().cookieMaxage = 604800
        } else {
            val attrMaxAge = getRequest().getAttribute(SSOConfig.SSO_COOKIE_MAXAGE)?.let {
                it as Int
            }
            if (attrMaxAge != null) {
                getRequest().removeAttribute(SSOConfig.SSO_COOKIE_MAXAGE)
            }
        }
        SSOHelper.setSSOCookie(getRequest(), getResponse(), SSOToken().also {
            it.uid = user.id.toString()
        }, true)
        return ResponseEntity.ok(ResponseDto())
    }

    @ApiOperation(value = "登出")
    @DeleteMapping("/logout")
    fun logout(): ResponseEntity<*> {
        SSOHelper.clearLogin(getRequest(), getResponse())
        return ResponseEntity.ok(ResponseDto())
    }
}