package com.gxzc.zen.web.sys.controller

import com.baomidou.kisso.annotation.Action
import com.baomidou.kisso.annotation.Login
import com.gxzc.zen.api.sys.model.SysDic
import com.gxzc.zen.api.sys.service.ISysDicService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.util.PaginationUtil
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.net.URI

/**
 * 系统字典 控制器
 * @author NorthLan
 * @date 2018/3/9
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("/sys/dic")
class SysDicController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(SysDicController::class.java)
    }

    @Autowired
    private lateinit var sysDicService: ISysDicService

    @GetMapping("/list")
    @Login(action = Action.Skip)
    @ZenResponseFilter(type = SysDic::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun getList(@RequestParam(required = false) keyword: String?,
                @RequestParam(required = false) searchOption: Int?): ResponseEntity<*> {
        var data: MutableList<SysDic> = sysDicService.getListCacheable()
        if (!keyword.isNullOrEmpty()) {
            data = data.filter {
                when (searchOption) {
                    1 -> run {
                        if (it.key != null) {
                            keyword!! in it.key!!
                        } else {
                            false
                        }
                    }
                    2 -> run {
                        if (it.label != null) {
                            keyword!! in it.label!!
                        } else {
                            false
                        }
                    }
                    else -> false
                }
            }.toMutableList()
        }

        return if (PaginationUtil.paginable(getRequest())) {
            // 分页
            ResponseEntity.ok(ResponseDto().apply {
                this.data = PaginationUtil.logicPaging(data, getRequest())
            })
        } else {
            ResponseEntity.ok(ResponseDto().apply {
                this.data = data
            })
        }
    }

    @GetMapping("{id}")
    @ZenResponseFilter(type = SysDic::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun getById(@PathVariable id: Long): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = sysDicService.selectById(id)
        })
    }

    @GetMapping("keys/{key}")
    @ZenResponseFilter(type = SysDic::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun getByKey(@PathVariable key: String): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = sysDicService.getListByKey(key)
        })
    }

    @PutMapping
    @ZenResponseFilter(type = SysDic::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun putDic(@RequestBody data: SysDic): ResponseEntity<*> {
        return if (data.id == null) {
            // insert
            sysDicService.insertCacheable(data)
            ResponseEntity.created(URI.create("/sys/dic/${data.id}")).body(ResponseDto()) // 201
        } else {
            // update
            ResponseEntity.ok(ResponseDto().apply {
                this.data = sysDicService.modify(data) // 200
            })
        }
    }

    @DeleteMapping("{id}")
    fun deleteDic(@PathVariable id: Long): ResponseEntity<*> {
        // 物理删除数据
        sysDicService.physicalDeleteCacheable(id)
        return ResponseEntity.ok(ResponseDto())
    }
}